@@ -1,4 +1,4 @@
-// SPDX-FileCopyrightText: 2017 The Qt Company Ltd.
+﻿// SPDX-FileCopyrightText: 2017 The Qt Company Ltd.
 // SPDX-FileCopyrightText: 2015-2024 Laurent Montel <montel@kde.org>
 // SPDX-FileCopyrightText: 2024 Carl Schwan <carl@carlschwan.eu>
 // SPDX-License-Identifier: BSD-3-Clause AND LGPL-2.0-or-later
@@ -10,22 +10,21 @@
 #include <KStandardShortcut>
 
 #include <QClipboard>
-#include <QDebug>
 #include <QFile>
 #include <QFileInfo>
-#include <QFileSelector>
 #include <QGuiApplication>
-#include <QMimeDatabase>
 #include <QPalette>
 #include <QQmlFile>
-#include <QQmlFileSelector>
-#include <QQuickTextDocument>
+#include <QTextBlock>
 #include <QTextCharFormat>
 #include <QTextDocument>
 #include <QTextList>
+#include <QTextTable>
 
 using namespace Qt::StringLiterals;
 
+constexpr int textMargin = 20;
+
 DocumentHandler::DocumentHandler(QObject *parent)
     : QObject(parent)
     , m_document(nullptr)
@@ -57,9 +56,7 @@ void DocumentHandler::setTextArea(QQuickItem *textArea)
 bool DocumentHandler::eventFilter(QObject *object, QEvent *event)
 {
     if (object == m_textArea && event->type() == QEvent::KeyPress) {
-        processKeyEvent(static_cast<QKeyEvent *>(event));
-        event->ignore();
-        return true;
+        return !processKeyEvent(static_cast<QKeyEvent *>(event));
     }
     return false;
 }
@@ -95,6 +92,7 @@ void DocumentHandler::setCursorPosition(int position)
 
     m_cursorPosition = position;
     reset();
+
     Q_EMIT cursorPositionChanged();
 }
 
@@ -290,6 +288,34 @@ QUrl DocumentHandler::fileUrl() const
     return m_fileUrl;
 }
 
+static void fixupTable(QTextFrame *frame)
+{
+    const auto children = frame->childFrames();
+    for (const auto child : children) {
+        if (auto table = dynamic_cast<QTextTable *>(child)) {
+            QTextTableFormat tableFormat;
+            tableFormat.setBorder(1);
+            const int numberOfColumns(table->columns());
+            QList<QTextLength> constrains;
+            constrains.reserve(numberOfColumns);
+            const QTextLength::Type type = QTextLength::PercentageLength;
+            const qreal length = 100; // 100% of window width
+
+            const QTextLength textlength(type, length / numberOfColumns);
+            for (int i = 0; i < numberOfColumns; ++i) {
+                constrains.append(textlength);
+            }
+            tableFormat.setColumnWidthConstraints(constrains);
+            tableFormat.setAlignment(Qt::AlignLeft);
+            tableFormat.setCellPadding(4);
+            tableFormat.setBorder(0.5);
+            tableFormat.setBorderCollapse(true);
+            tableFormat.setTopMargin(textMargin);
+            table->setFormat(tableFormat);
+        }
+    }
+}
+
 void DocumentHandler::load(const QUrl &fileUrl)
 {
     if (fileUrl == m_fileUrl) {
@@ -309,9 +335,42 @@ void DocumentHandler::load(const QUrl &fileUrl)
     }
 
     QByteArray data = file.readAll();
+    QString content;
+    m_frontMatter = QString{};
+
+    if (data.startsWith("---") || data.startsWith("***") || data.startsWith("+++")) {
+        // we have a front matter
+        QTextStream stream(&data);
+        QString firstLine;
+        int line = 0;
+        QString lineContent;
+        bool foundEnd = false;
+
+        while (stream.readLineInto(&lineContent)) {
+            if (line == 0) {
+                firstLine = lineContent;
+                m_frontMatter += lineContent + u'\n';
+                line++;
+                continue;
+            }
+
+            line++;
+            if (!foundEnd) {
+                m_frontMatter += lineContent + u'\n';
+                if (lineContent == firstLine) {
+                    foundEnd = true;
+                }
+            } else {
+                content += lineContent + u'\n';
+            }
+        }
+    } else {
+        content = QString::fromUtf8(data);
+    }
+
     if (QTextDocument *doc = textDocument()) {
         doc->setBaseUrl(QUrl(fileUrl).adjusted(QUrl::RemoveFilename));
-        Q_EMIT loaded(QString::fromUtf8(data), Qt::MarkdownText);
+        Q_EMIT loaded(content, Qt::MarkdownText);
         doc->setModified(false);
     }
 
@@ -322,9 +381,18 @@ void DocumentHandler::load(const QUrl &fileUrl)
         for (it = currentBlock.begin(); !it.atEnd(); ++it) {
             QTextFragment fragment = it.fragment();
             if (fragment.isValid()) {
+                const int pos = fragment.position();
+
+                QTextCursor cursor(textDocument());
+                cursor.setPosition(pos);
+                if (!cursor.currentList() || cursor.currentList()->item(0) == currentBlock) {
+                    QTextBlockFormat format;
+                    format.setTopMargin(textMargin);
+                    cursor.mergeBlockFormat(format);
+                }
+
                 QTextImageFormat imageFormat = fragment.charFormat().toImageFormat();
                 if (imageFormat.isValid()) {
-                    int pos = fragment.position();
                     if (!cursorPositionsToSkip.contains(pos)) {
                         QTextCursor cursor(textDocument());
                         cursor.setPosition(pos);
@@ -345,6 +413,12 @@ void DocumentHandler::load(const QUrl &fileUrl)
         currentBlock = currentBlock.next();
     }
 
+    fixupTable(textDocument()->rootFrame());
+
+    QTextCursor cursor = textCursor();
+    cursor.movePosition(QTextCursor::End);
+    moveCursor(cursor.position());
+
     reset();
 }
 
@@ -360,6 +434,9 @@ void DocumentHandler::saveAs(const QUrl &fileUrl)
         Q_EMIT error(tr("Cannot save: ") + file.errorString() + u' ' + fileUrl.toLocalFile());
         return;
     }
+    if (!m_frontMatter.isEmpty()) {
+        file.write(m_frontMatter.toUtf8());
+    }
     file.write(doc->toMarkdown().toUtf8());
     file.close();
 
@@ -647,6 +724,45 @@ void DocumentHandler::insertImage(const QUrl &url)
     textCursor().insertHtml(u"<img width=\"500\" src=\""_s + url.path() + u"\"\\>"_s);
 }
 
+void DocumentHandler::insertTable(int rows, int columns)
+{
+    QString htmlText;
+
+    QTextCursor cursor = textCursor();
+    QTextTableFormat tableFormat;
+    tableFormat.setBorder(1);
+    const int numberOfColumns(columns);
+    QList<QTextLength> constrains;
+    constrains.reserve(numberOfColumns);
+    const QTextLength::Type type = QTextLength::PercentageLength;
+    const int length = 100; // 100% of window width
+
+    const QTextLength textlength(type, length / numberOfColumns);
+    for (int i = 0; i < numberOfColumns; ++i) {
+        constrains.append(textlength);
+    }
+    tableFormat.setColumnWidthConstraints(constrains);
+    tableFormat.setAlignment(Qt::AlignLeft);
+    tableFormat.setCellSpacing(0);
+    tableFormat.setCellPadding(4);
+    tableFormat.setBorderCollapse(true);
+    tableFormat.setBorder(0.5);
+    tableFormat.setTopMargin(20);
+
+    Q_ASSERT(cursor.document());
+    QTextTable *table = cursor.insertTable(rows, numberOfColumns, tableFormat);
+
+    // fill table with whitespace
+    for (int i = 0, rows = table->rows(); i < rows; i++) {
+        for (int j = 0, columns = table->columns(); j < columns; j++) {
+            auto cell = table->cellAt(i, j);
+            Q_ASSERT(cell.isValid());
+            cell.firstCursorPosition().insertText(u" "_s);
+        }
+    }
+    return;
+}
+
 void DocumentHandler::setCheckable(bool add)
 {
     QTextBlockFormat fmt;
@@ -675,11 +791,10 @@ bool DocumentHandler::checkable() const
         || textCursor().blockFormat().marker() == QTextBlockFormat::MarkerType::Checked;
 }
 
-void DocumentHandler::evaluateReturnKeySupport(QKeyEvent *event)
+bool DocumentHandler::evaluateReturnKeySupport(QKeyEvent *event)
 {
     if (event->key() != Qt::Key_Return) {
-        evaluateListSupport(event);
-        return;
+        return evaluateListSupport(event);
     }
 
     QTextCursor cursor = textCursor();
@@ -726,12 +841,13 @@ void DocumentHandler::evaluateReturnKeySupport(QKeyEvent *event)
             cursor.movePosition(QTextCursor::StartOfBlock);
             setCursorPosition(cursor.position());
         }
+        return true;
     } else {
-        evaluateListSupport(event);
+        return evaluateListSupport(event);
     }
 }
 
-void DocumentHandler::evaluateListSupport(QKeyEvent *event)
+bool DocumentHandler::evaluateListSupport(QKeyEvent *event)
 {
     bool handled = false;
     if (textCursor().currentList()) {
@@ -775,23 +891,150 @@ void DocumentHandler::evaluateListSupport(QKeyEvent *event)
         } else if (event->key() == Qt::Key_Down && isControlClicked) {
             moveCursorBeginUpDown(false);
             event->accept();
-        } else {
-            m_textArea->keyPressEvent(event);
+        }
+        return true;
+    }
+    return true;
+}
+
+void DocumentHandler::slotKeyPressed(int key)
+{
+    if (key == Qt::Key_Space) {
+        const auto blockText = textCursor().block().text();
+
+        // Automatic block transformation to header
+        if (blockText.startsWith(u'#')) {
+            int i = 0;
+            while (blockText.length() > i && i < 6 && blockText[i] == u'#') {
+                i++;
+            }
+
+            auto cursor = textCursor();
+            cursor.beginEditBlock();
+            setHeadingLevel(i);
+            cursor.movePosition(QTextCursor::Left, QTextCursor::KeepAnchor, i + 1);
+            cursor.deleteChar();
+            cursor.endEditBlock();
+        }
+
+        // Automatic block transformation to list
+        if (blockText.startsWith(u"* ") || blockText.startsWith(u"- ")) {
+            auto cursor = textCursor();
+            cursor.beginEditBlock();
+            setListStyle(1);
+            cursor.movePosition(QTextCursor::Left, QTextCursor::KeepAnchor, 2);
+            cursor.deleteChar();
+            cursor.endEditBlock();
         }
     }
 
+    if (key != Qt::Key_Return) {
+        auto cursor = textCursor();
+        const auto blockText = cursor.block().text().left(cursor.positionInBlock() - 1);
+
+        auto transform = [this, &cursor, &blockText](const QString &symbol, const QTextCharFormat &format) {
+            const auto firstSymbolsInBlock = blockText.indexOf(symbol);
+            const auto symbolSize = symbol.length();
+
+            if (symbolSize == 1 && blockText.indexOf(symbol + symbol) == firstSymbolsInBlock) {
+                // Prefer matching with either **text** or __text__ instead of just **text* or __text_
+                return;
+            }
+
+            if (firstSymbolsInBlock == -1 || !blockText.endsWith(symbol) || (firstSymbolsInBlock + symbolSize + 2 >= cursor.positionInBlock())) {
+                return;
+            }
+
+            cursor.beginEditBlock();
+            cursor.movePosition(QTextCursor::Left, QTextCursor::MoveAnchor, 1);
+
+            // delete the last instance of the symbol
+            cursor.movePosition(QTextCursor::Left, QTextCursor::KeepAnchor, symbolSize);
+            cursor.deleteChar();
+
+            // select the text and bold it
+            const auto selectionSize = cursor.positionInBlock() - (firstSymbolsInBlock + symbolSize);
+            cursor.movePosition(QTextCursor::Left, QTextCursor::KeepAnchor, selectionSize);
+
+            cursor.mergeCharFormat(format);
+
+            // delete the first instance of the symbol
+            cursor.clearSelection();
+            cursor.movePosition(QTextCursor::Left, QTextCursor::KeepAnchor, symbolSize);
+            cursor.deleteChar();
+
+            // move back to initial position and font weight
+            cursor.movePosition(QTextCursor::Right, QTextCursor::MoveAnchor, selectionSize);
+
+            QTextCharFormat normalFormat;
+            normalFormat.setFontWeight(QFont::Normal);
+            normalFormat.setFontUnderline(false);
+            normalFormat.setFontStrikeOut(false);
+            cursor.mergeCharFormat(normalFormat);
+
+            cursor.endEditBlock();
+            Q_EMIT cursorPositionChanged();
+        };
+
+        // bold
+        QTextCharFormat boldFormat;
+        boldFormat.setFontWeight(QFont::Bold);
+        transform(u"**"_s, boldFormat);
+        transform(u"__"_s, boldFormat);
+
+        // italic
+        QTextCharFormat italicFormat;
+        italicFormat.setFontItalic(true);
+        transform(u"*"_s, italicFormat);
+
+        // underline
+        QTextCharFormat underlineFormat;
+        underlineFormat.setFontUnderline(true);
+        transform(u"_"_s, underlineFormat);
+
+        // strikethrough
+        QTextCharFormat strikethroughFormat;
+        strikethroughFormat.setFontStrikeOut(true);
+        transform(u"~~"_s, strikethroughFormat);
+    }
+
     // Match the behavior of office suites: newline after header switches to normal text
-    if ((event->key() == Qt::Key_Return) && (textCursor().blockFormat().headingLevel() > 0) && (textCursor().atBlockEnd())) {
+    if ((key == Qt::Key_Return) && (textCursor().blockFormat().headingLevel() > 0) && (textCursor().atBlockEnd())) {
         // it should be undoable together with actual "return" keypress
         textCursor().joinPreviousEditBlock();
         setHeadingLevel(0);
         textCursor().endEditBlock();
+        Q_EMIT cursorPositionChanged();
     }
 
     if (textCursor().currentList()) {
-        m_nestedListHelper.handleAfterKeyPressEvent(event, textCursor());
+        if ((key != Qt::Key_Backspace) && (key != Qt::Key_Return)) {
+            return;
+        }
+
+        auto cursor = textCursor();
+        QTextBlock currentBlock = cursor.block();
+        if (cursor.currentList()->count() == cursor.currentList()->itemNumber(currentBlock) + 1) {
+            if (cursor.currentList()->count() > 1 && cursor.currentList()->itemNumber(currentBlock)) {
+                if (currentBlock.previous().text().isEmpty()) {
+                    cursor.joinPreviousEditBlock();
+                    QTextBlockFormat bfmt;
+                    bfmt.setTopMargin(textMargin);
+                    bfmt.setBottomMargin(0);
+                    textCursor().setBlockFormat(bfmt);
+                    textCursor().endEditBlock();
+                    return;
+                }
+            }
+        }
+
+        cursor.joinPreviousEditBlock();
+        QTextBlockFormat bfmt = textCursor().block().blockFormat();
+        bfmt.setTopMargin(cursor.block().previous().textList() == nullptr ? textMargin : 0);
+        bfmt.setBottomMargin(0);
+        textCursor().setBlockFormat(bfmt);
+        textCursor().endEditBlock();
     }
-    Q_EMIT cursorPositionChanged();
 }
 
 bool DocumentHandler::processKeyEvent(QKeyEvent *e)
@@ -800,18 +1043,34 @@ bool DocumentHandler::processKeyEvent(QKeyEvent *e)
         && textCursor().block().layout()->lineForTextPosition(textCursor().position()).lineNumber() == 0) {
         textCursor().clearSelection();
         Q_EMIT focusUp();
-    } else if (e->key() == Qt::Key_Backtab && e->modifiers() == Qt::ShiftModifier) {
-        textCursor().clearSelection();
-        Q_EMIT focusUp();
     } else {
-        evaluateReturnKeySupport(e);
+        QTextTable *table = textCursor().currentTable();
+        const bool isTable = (table != nullptr);
+        if (isTable && (e->key() == Qt::Key_Backtab || e->key() == Qt::Key_Tab)) {
+            // navigate to previous or next table cell
+            auto cursor = textCursor();
+            auto ok = cursor.movePosition(e->key() == Qt::Key_Tab ? QTextCursor::NextCell : QTextCursor::PreviousCell);
+            if (!ok) {
+                ok = cursor.movePosition(e->key() == Qt::Key_Tab ? QTextCursor::NextBlock : QTextCursor::PreviousBlock);
+                moveCursor(cursor.position());
+            } else {
+                cursor.movePosition(QTextCursor::StartOfLine);
+                const int start = cursor.position();
+                cursor.movePosition(QTextCursor::EndOfLine);
+                const int end = cursor.position();
+                selectCursor(start, end);
+            }
+            return false;
+        } else {
+            return evaluateReturnKeySupport(e);
+        }
     }
     return true;
 }
 
 bool DocumentHandler::handleShortcut(QKeyEvent *event)
 {
-    const int key = event->key() | event->modifiers();
+    const QKeySequence key = event->modifiers() | (Qt::Key)event->key();
 
     if (KStandardShortcut::copy().contains(key)) {
         copy();
@@ -837,32 +1096,32 @@ bool DocumentHandler::handleShortcut(QKeyEvent *event)
     } else if (KStandardShortcut::backwardWord().contains(key)) {
         QTextCursor cursor = textCursor();
         cursor.movePosition(QTextCursor::PreviousWord);
-        setCursorPosition(cursor.position());
+        moveCursor(cursor.position());
         return true;
     } else if (KStandardShortcut::forwardWord().contains(key)) {
         QTextCursor cursor = textCursor();
         cursor.movePosition(QTextCursor::NextWord);
-        setCursorPosition(cursor.position());
+        moveCursor(cursor.position());
         return true;
     } else if (KStandardShortcut::begin().contains(key)) {
         QTextCursor cursor = textCursor();
         cursor.movePosition(QTextCursor::Start);
-        setCursorPosition(cursor.position());
+        moveCursor(cursor.position());
         return true;
     } else if (KStandardShortcut::end().contains(key)) {
         QTextCursor cursor = textCursor();
         cursor.movePosition(QTextCursor::End);
-        setCursorPosition(cursor.position());
+        moveCursor(cursor.position());
         return true;
     } else if (KStandardShortcut::beginningOfLine().contains(key)) {
         QTextCursor cursor = textCursor();
         cursor.movePosition(QTextCursor::StartOfLine);
-        setCursorPosition(cursor.position());
+        moveCursor(cursor.position());
         return true;
     } else if (KStandardShortcut::endOfLine().contains(key)) {
         QTextCursor cursor = textCursor();
         cursor.movePosition(QTextCursor::EndOfLine);
-        setCursorPosition(cursor.position());
+        moveCursor(cursor.position());
         return true;
         //} else if (searchSupport() && KStandardShortcut::find().contains(key)) {
         //    Q_EMIT findText();
@@ -887,7 +1146,7 @@ bool DocumentHandler::handleShortcut(QKeyEvent *event)
             cursor.movePosition(QTextCursor::EndOfBlock, QTextCursor::KeepAnchor);
         }
         cursor.removeSelectedText();
-        setCursorPosition(cursor.position());
+        moveCursor(cursor.position());
         return true;
     }
 
@@ -975,3 +1234,5 @@ void DocumentHandler::deleteWordForward()
 {
     deleteWord(textCursor(), QTextCursor::WordRight);
 }
+
+#include "moc_documenthandler.cpp"
