@@ -4,21 +4,27 @@
 */
 
 #include <KAboutData>
+#include <KCrash>
 #include <KLocalizedContext>
 #include <KLocalizedString>
 #include <QApplication>
 #include <QIcon>
 #include <QQmlApplicationEngine>
+#include <QQmlContext>
 #include <QQuickStyle>
 #include <QUrl>
-#include <QtQml>
 
 #include "../marknote-version.h"
+#include "config.h"
 
 #ifdef Q_OS_WINDOWS
 #include <Windows.h>
 #endif
 
+#ifdef WITH_BREEZEICONS_LIB
+#include <BreezeIcons>
+#endif
+
 using namespace Qt::Literals::StringLiterals;
 
 int main(int argc, char *argv[])
@@ -27,12 +33,19 @@ int main(int argc, char *argv[])
     QGuiApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
 #endif
 
+    QIcon::setFallbackThemeName(u"breeze"_s);
     QApplication app(argc, argv);
     // Default to org.kde.desktop style unless the user forces another style
     if (qEnvironmentVariableIsEmpty("QT_QUICK_CONTROLS_STYLE")) {
         QQuickStyle::setStyle(QStringLiteral("org.kde.desktop"));
     }
 
+    KCrash::initialize();
+
+#ifdef WITH_BREEZEICONS_LIB
+    BreezeIcons::initIcons();
+#endif
+
 #ifdef Q_OS_WINDOWS
     if (AttachConsole(ATTACH_PARENT_PROCESS)) {
         freopen("CONOUT$", "w", stdout);
@@ -44,17 +57,28 @@ int main(int argc, char *argv[])
     font.setPointSize(10);
     app.setFont(font);
 #endif
+
+#ifdef Q_OS_MACOS
+    QApplication::setStyle(QStringLiteral("breeze"));
+#endif
+
     KLocalizedString::setApplicationDomain("marknote");
 
+    QCommandLineParser parser;
+
     QGuiApplication::setWindowIcon(QIcon::fromTheme(QStringLiteral("org.kde.marknote")));
 
     KAboutData about(QStringLiteral("marknote"),
-                     i18n("Marknote"),
+                     i18nc("Application name", "Marknote"),
                      QStringLiteral(MARKNOTE_VERSION_STRING),
                      i18n("Note taking application"),
                      KAboutLicense::GPL_V2,
                      i18n("© 2023 Mathis Brüchert"));
-    about.addAuthor(i18n("Mathis Brüchert"), i18n("Maintainer"), QStringLiteral("mbb-mail@gmx.de"));
+    about.addAuthor(i18n("Mathis Brüchert"),
+                    i18n("Maintainer"),
+                    QStringLiteral("mbb@kaidan.im"),
+                    QStringLiteral("https://invent.kde.org/mbruchert"),
+                    QUrl(QStringLiteral("https://gravatar.com/avatar/f9c35f242fe79337bf8746ca9fccc189?size=256.png")));
     about.addAuthor(i18n("Carl Schwan"),
                     i18n("Maintainer"),
                     QStringLiteral("carl@carlschwan.eu"),
@@ -64,13 +88,31 @@ int main(int argc, char *argv[])
 
     KAboutData::setApplicationData(about);
 
+    about.setupCommandLine(&parser);
+    parser.process(app);
+    about.processCommandLine(&parser);
+
     QQmlApplicationEngine engine;
     engine.rootContext()->setContextObject(new KLocalizedContext(&engine));
-    engine.loadFromModule(u"org.kde.marknote"_s, u"Main"_s);
+
+    if (parser.positionalArguments().length() > 0) {
+        const auto path = parser.positionalArguments()[0];
+        if (QFile::exists(path)) {
+            engine.rootContext()->setContextProperty(u"cliNoteName"_s, path.split(QLatin1Char('/')).last().replace(u".md"_s, QString{}));
+            engine.rootContext()->setContextProperty(u"cliNoteFullPath"_s, QUrl::fromLocalFile(path));
+        }
+        engine.loadFromModule(u"org.kde.marknote"_s, u"MainEditor"_s);
+    } else {
+        engine.loadFromModule(u"org.kde.marknote"_s, u"Main"_s);
+    }
 
     if (engine.rootObjects().isEmpty()) {
         return -1;
     }
 
+    QObject::connect(QApplication::instance(), &QCoreApplication::aboutToQuit, QApplication::instance(), [&engine] {
+        engine.singletonInstance<Config *>("org.kde.marknote", "Config")->save();
+    });
+
     return app.exec();
 }
